// Bookmark Backup
// By Pike

// Contributor(s):
// Nickolay Ponomarev (Multiple file backup code)

var Settings = {
	
	onLoad: function()
	{
		// Setup object properties
		this.backupService = Components.classes['@pikey.me.uk/backup-service;1'].getService(Components.interfaces.iBackupService);
		this.prefBranch = Components.classes['@mozilla.org/preferences-service;1'].getService(Components.interfaces.nsIPrefService).getBranch('bookmarkbackup.');
		this.localeStrings = document.getElementById('strings');
		this.whereField = document.getElementById('where');
		this.locationField = document.getElementById('location');
		this.gridElem = document.getElementById('backupGroupsGrid');
		this.otherFilesField = document.getElementById('otherFiles');
		
		// Backup location
		var location;
		try
		{
			location = this.prefBranch.getComplexValue('location', Components.interfaces.nsILocalFile).path;
		}
		catch (err) {}
		if (location)
		{
			this.whereField.selectedItem = document.getElementById('custom');
			this.locationField.value = location;
		}
		else
		{
			this.whereField.selectedItem = document.getElementById('default');
		}
		
		// Backup groups
		var checkboxCreator = new CheckboxCreator(this.gridElem);
		var backupGroups = this.backupService.backupGroups;
		var chosenGroups = this.prefBranch.getComplexValue('chosenGroups', Components.interfaces.nsISupportsString).data;
		chosenGroups = chosenGroups.split('|').sort();
		while (backupGroups.hasMore())
		{
			var group = backupGroups.getNext();
			if (group != '_default')
			{
				var curBox = checkboxCreator.addBox(group, this.getLabel(group), this.getTooltip(group));
				if (chosenGroups.indexOf(group) != -1)
				{
					curBox.checked = true;
				}
			}
		}
		if (!checkboxCreator.boxCount())
		{
			this.gridElem.parent.style.display = 'none';
		}
		
		// Other files
		var chosenFiles = this.prefBranch.getComplexValue('chosenFiles', Components.interfaces.nsISupportsString).data;
		this.otherFilesField.value = chosenFiles;
	},
	
	onAccept: function()
	{
		// Backup location
		var location = this.trimString(this.locationField.value);
		if (this.whereField.selectedItem.id == 'custom')
		{
			this.setFilePref(location, 'location');
		}
		else
		{
			this.setFilePref(null, 'location');
		}
		
		// Backup groups
		var chosenGroups = [];
		var checkboxes = this.gridElem.getElementsByTagName('checkbox');
		for (var i = 0; i < checkboxes.length; i++)
		{
			if (checkboxes[i].checked)
			{
				chosenGroups.push(checkboxes[i].id);
			}
		}
		this.setArrayPref(chosenGroups, 'chosenGroups');
		
		// Other files
		var chosenFiles = [];
		var otherFiles = this.otherFilesField.value.split('|');
		for (var j = 0; j < otherFiles.length; j++)
		{
			otherFiles[j] = this.trimString(otherFiles[j]);
			if (otherFiles[j])
			{
				chosenFiles.push(otherFiles[j]);
			}
		}
		this.setArrayPref(chosenFiles, 'chosenFiles');
	},
	
	onSelect: function(selected)
	{
		var browseButton = document.getElementById('browse');
		if (selected.id == 'default')
		{
			this.locationField.disabled = true;
			browseButton.disabled = true;
		}
		else
		{
			this.locationField.disabled = false;
			browseButton.disabled = false;
		}
	},
	
	browse: function()
	{
		var filePicker = Components.classes['@mozilla.org/filepicker;1'].createInstance(Components.interfaces.nsIFilePicker);
		filePicker.init(window, this.localeStrings.getString('selectBackupFolder'), filePicker.modeGetFolder);
		try
		{
			var location = this.trimString(this.locationField.value);
			if (location)
			{
				var currentLocation = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
				currentLocation.initWithPath(location);
				filePicker.displayDirectory = currentLocation;
			}
		}
		catch (err) {}
		filePicker.appendFilters(filePicker.filterAll);
		var result = filePicker.show();
		if (result == filePicker.returnOK)
		{
			this.locationField.value = filePicker.file.path;
		}
	},
	
	selectAll: function()
	{
		var checkboxes = this.gridElem.getElementsByTagName('checkbox');
		for (var i = 0; i < checkboxes.length; i++)
		{
			checkboxes[i].checked = true;
		}
	},
	
	selectNone: function()
	{
		var checkboxes = this.gridElem.getElementsByTagName('checkbox');
		for (var i = 0; i < checkboxes.length; i++)
		{
			checkboxes[i].checked = false;
		}
	},
	
	getLabel: function(id)
	{
		var labelText;
		try
		{
			// Use a try/catch here because if a custom checkbox has been added there won't be a
			// matching string in the locale
			labelText = this.localeStrings.getString(id);
		}
		catch (err)
		{
			labelText = id;
		}
		return labelText;
	},
	
	getTooltip: function(id)
	{
		var fileList = this.backupService.getBackupGroup(id, {}).join(', ');
		return this.localeStrings.getFormattedString('backsUp', [fileList]);
	},
	
	setFilePref: function(filePath, prefName)
	{
		var success = false;
		if (filePath)
		{
			try
			{
				var file = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
				file.initWithPath(filePath);
				this.prefBranch.setComplexValue(prefName, Components.interfaces.nsILocalFile, file);
				success = true;
			}
			catch (err) {}
		}
		if (!success)
		{
			try
			{
				this.prefBranch.clearUserPref(prefName);
			}
			catch (err) {}
		}
	},
	
	// XXX - FIXME: This method is duplicated in both the component and the settings code
	setArrayPref: function(list, prefName)
	{
		// Remove duplicates
		var prunedList = [];
		var lastValue;
		list.sort();
		for (var i = 0; i < list.length; i++)
		{
			if (!i || list[i] != lastValue)
			{
				prunedList.push(list[i]);
				lastValue = list[i];
			}
		}
		
		// Save pref
		if (prunedList.length)
		{
			var listStr = Components.classes['@mozilla.org/supports-string;1'].createInstance(Components.interfaces.nsISupportsString);
			listStr.data = prunedList.join('|');
			this.prefBranch.setComplexValue(prefName, Components.interfaces.nsISupportsString, listStr);
		}
		else
		{
			try
			{
				this.prefBranch.clearUserPref(prefName);
			}
			catch (err) {}
		}
	},
	
	trimString: function(string)
	{
		return string.replace(/(^\s+)|(\s+$)/g, '');
	}
};

function CheckboxCreator(gridElem)
{
	this._rowsElem = gridElem.getElementsByTagName('rows')[0];
	this._numCols = Math.max(gridElem.getElementsByTagName('column').length, 1);
	this._numBoxes = 0;
}

CheckboxCreator.prototype = {

	addBox: function(id, label, tooltip)
	{
		// Create checkbox
		var newCheckbox = document.createElement('checkbox');
		newCheckbox.id = id;
		newCheckbox.setAttribute('label', label);
		newCheckbox.setAttribute('tooltiptext', tooltip);
		
		// Add checkbox to last row
		var lastRow;
		if (this._numBoxes % this._numCols === 0)
		{
			lastRow = document.createElement('row');
			this._rowsElem.appendChild(lastRow);
		}
		else
		{
			lastRow = this._rowsElem.lastChild;
		}
		lastRow.appendChild(newCheckbox);
		this._numBoxes++;
		
		return newCheckbox;
	},
	
	boxCount: function()
	{
		return this._numBoxes;
	}
};
